package com.guwan.backend.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.guwan.backend.common.Result;
import com.guwan.backend.dto.video.VideoDTO;
import com.guwan.backend.service.VideoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

@Slf4j
@Tag(name = "视频管理", description = "视频相关接口")
@RestController
@RequestMapping("/api/videos")
@RequiredArgsConstructor
public class VideoController {

    private final VideoService videoService;


    @Operation(summary = "上传视频", description = "上传视频文件并返回视频信息")
    @SecurityRequirement(name = "bearer-jwt")
    @PostMapping("/upload")
    public Result<VideoDTO> uploadVideo(
            @Parameter(description = "视频文件") @RequestParam("file") MultipartFile file,
            @Parameter(description = "视频标题") @RequestParam("title") String title,
            @Parameter(description = "视频描述") @RequestParam("description") String description,
            @Parameter(description = "视频标签，多个用逗号分隔") @RequestParam(value = "tags", required = false) String tags) {
        try {
            VideoDTO video = videoService.uploadVideo(file, title, description, tags);
            return Result.success(video);
        } catch (Exception e) {
            log.error("上传视频失败", e);
            return Result.error(e.getMessage());
        }
    }

    @Operation(summary = "更新视频信息")
    @SecurityRequirement(name = "bearer-jwt")
    @PutMapping("/{id}")
    public Result<VideoDTO> updateVideo(
            @Parameter(description = "视频ID") @PathVariable Long id,
            @RequestBody VideoDTO videoDTO) {
        try {
            videoDTO.setId(id);
            return Result.success(videoService.updateVideo(videoDTO));
        } catch (Exception e) {
            log.error("更新视频失败", e);
            return Result.error(e.getMessage());
        }
    }

    @Operation(summary = "删除视频")
    @SecurityRequirement(name = "bearer-jwt")
    @DeleteMapping("/{id}")
    public Result<Void> deleteVideo(
            @Parameter(description = "视频ID") @PathVariable Long id) {
        try {
            videoService.deleteVideo(id);
            return Result.success();
        } catch (Exception e) {
            log.error("删除视频失败", e);
            return Result.error(e.getMessage());
        }
    }

    @Operation(summary = "获取视频详情")
    @GetMapping("/{id}")
    public Result<VideoDTO> getVideo(
            @Parameter(description = "视频ID") @PathVariable Long id) {
        try {
            VideoDTO video = videoService.getVideoById(id);
            if (video == null) {
                return Result.notFound("视频不存在");
            }
            return Result.success(video);
        } catch (Exception e) {
            log.error("获取视频失败", e);
            return Result.error(e.getMessage());
        }
    }

    @Operation(summary = "获取视频列表", description = "支持分页和关键词搜索")
    @GetMapping
    public Result<IPage<VideoDTO>> getVideoList(
            @Parameter(description = "页码") @RequestParam(defaultValue = "1") Integer pageNum,
            @Parameter(description = "每页条数") @RequestParam(defaultValue = "10") Integer pageSize,
            @Parameter(description = "搜索关键词") @RequestParam(required = false) String keyword) {
        try {
            return Result.success(videoService.getVideoList(pageNum, pageSize, keyword));
        } catch (Exception e) {
            log.error("获取视频列表失败", e);
            return Result.error(e.getMessage());
        }
    }

    @Operation(summary = "增加视频观看次数")
    @PostMapping("/{id}/view")
    public Result<Void> incrementViewCount(
            @Parameter(description = "视频ID") @PathVariable Long id) {
        try {
            videoService.incrementViewCount(id);
            return Result.success();
        } catch (Exception e) {
            log.error("增加观看次数失败", e);
            return Result.error(e.getMessage());
        }
    }

    @Operation(summary = "视频点赞/取消点赞")
    @SecurityRequirement(name = "bearer-jwt")
    @PostMapping("/{id}/like")
    public Result<Void> toggleLike(
            @Parameter(description = "视频ID") @PathVariable Long id) {
        try {
            videoService.toggleLike(id);
            return Result.success();
        } catch (Exception e) {
            log.error("点赞失败", e);
            return Result.error(e.getMessage());
        }
    }




} 