package com.guwan.backend.util;

import io.minio.*;
import io.minio.http.Method;
import io.minio.messages.DeleteError;
import io.minio.messages.DeleteObject;
import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FilenameUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Base64;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.TimeUnit;



@Slf4j
@Component
@RequiredArgsConstructor
public class MinioUtil {

    private final MinioClient minioClient;

    /**
     * 创建存储桶
     */
    public void createBucket(String bucketName) {
        try {
            boolean found = minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
            if (!found) {
                minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
            }
        } catch (Exception e) {
            log.error("创建存储桶失败", e);
            throw new RuntimeException("创建存储桶失败", e);
        }
    }

    /**
     * 上传文件
     */
    public String uploadFile(String bucketName, MultipartFile file) {
        try {
            String fileName = generateFileName(file.getOriginalFilename());
            InputStream inputStream = file.getInputStream();
            minioClient.putObject(
                PutObjectArgs.builder()
                    .bucket(bucketName)
                    .object(fileName)
                    .stream(inputStream, file.getSize(), -1)
                    .contentType(file.getContentType())
                    .build()
            );
            return fileName;
        } catch (Exception e) {
            log.error("上传文件失败", e);
            throw new RuntimeException("上传文件失败", e);
        }
    }

    /**
     * 上传Base64图片
     */
    public String uploadBase64Image(String bucketName, String base64Image, String folder) {
        try {
            String[] parts = base64Image.split(",");
            String imageData = parts[1];
            byte[] bytes = Base64.getDecoder().decode(imageData);
            
            String fileName = folder + "/" + UUID.randomUUID() + ".jpg";
            ByteArrayInputStream inputStream = new ByteArrayInputStream(bytes);
            
            minioClient.putObject(
                PutObjectArgs.builder()
                    .bucket(bucketName)
                    .object(fileName)
                    .stream(inputStream, bytes.length, -1)
                    .contentType("image/jpeg")
                    .build()
            );
            return fileName;
        } catch (Exception e) {
            log.error("上传Base64图片失败", e);
            throw new RuntimeException("上传Base64图片失败", e);
        }
    }

    /**
     * 删除文件
     */
    public void deleteFile(String bucketName, String fileName) {
        try {
            minioClient.removeObject(
                RemoveObjectArgs.builder()
                    .bucket(bucketName)
                    .object(fileName)
                    .build()
            );
        } catch (Exception e) {
            log.error("删除文件失败", e);
            throw new RuntimeException("删除文件失败", e);
        }
    }

    /**
     * 批量删除文件
     */
    public void deleteFiles(String bucketName, List<String> fileNames) {
        try {
            List<DeleteObject> objects = fileNames.stream()
                .map(DeleteObject::new)
                .toList();
            
            Iterable<Result<DeleteError>> results = minioClient.removeObjects(
                RemoveObjectsArgs.builder()
                    .bucket(bucketName)
                    .objects(objects)
                    .build()
            );
            
            for (Result<DeleteError> result : results) {
                DeleteError error = result.get();
                log.error("删除文件失败: {}", error.message());
            }
        } catch (Exception e) {
            log.error("批量删除文件失败", e);
            throw new RuntimeException("批量删除文件失败", e);
        }
    }

    /**
     * 获取文件访问URL
     */
    public String getFileUrl(String bucketName, String fileName) {
        try {
            return minioClient.getPresignedObjectUrl(
                GetPresignedObjectUrlArgs.builder()
                    .method(Method.GET)
                    .bucket(bucketName)
                    .object(fileName)
                    .expiry(7, TimeUnit.DAYS)
                    .build()
            );
        } catch (Exception e) {
            log.error("获取文件访问URL失败", e);
            throw new RuntimeException("获取文件访问URL失败", e);
        }
    }

    /**
     * 生成文件名
     */
    private String generateFileName(String originalFilename) {
        String extension = FilenameUtils.getExtension(originalFilename);
        return UUID.randomUUID() + "." + extension;
    }

    // 初始化时创建视频桶
    @PostConstruct
    public void init() {
        try {
            createBucket("videos");
        } catch (Exception e) {
            log.error("创建视频存储桶失败", e);
        }
    }
} 